#ifndef tst_h__
#define tst_h__
/*
 * tst.h - ternary search trees
 * Copyright (C) 1998, 1999, 2000 Rex Feany <laeos@laeos.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * @(#)tst.h 1.1
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

/* number of buckets to use. Please, be a power of two! */
#define XP_TST_NBUCKETS	32

/* Given a string, choose the right bucket */
#define XP_TST_INDEX(x)   ((unsigned int)(tolower(x)&(XP_TST_NBUCKETS-1)))

#define XP_TST_BUCKET(th, str)	((th)->th_head[XP_TST_INDEX(*(str))])

/* each node of a tst */
struct xp_tst_node {
	unsigned char	   tn_split;
	struct xp_tst_node *tn_low;
	struct xp_tst_node *tn_eq;
	struct xp_tst_node *tn_hi;
	void		   *tn_data;
};

/* The head of a ternary search tree */
struct xp_tst_head {
	struct xp_tst_node   *th_head[XP_TST_NBUCKETS-1];
};

/* Used for the return array of tst_partial */
struct xp_tst_array {
	void *data;
};

/* What the outside uses */
typedef struct xp_tst_head XP_TST_HEAD;
typedef struct xp_tst_array XP_TST_ARRAY;

/* Most of these routines are recursive. */

/* initilize a new tree (xp_tst_init.c) */
void xp_tst_init (XP_TST_HEAD *tst);

/* Insert a node, fail if one with the same key exists */
int xp_tst_insert(XP_TST_HEAD *tst, const char *key, void *tst_node);

/* Insert a node, replace if one exists */
void * xp_tst_replace(XP_TST_HEAD *tst, const char *key, void *data);

/* Remove a node (xp_tst_remove.c) */
void * xp_tst_remove (XP_TST_HEAD *tst, const char *key);

/* Find a specific node */
void * xp_tst_find(XP_TST_HEAD *tst, const char *key);

/* Partial search. Return a list of things found.
 * pos == where to start the list, len == how many to return. */
XP_TST_ARRAY * xp_tst_partial(XP_TST_HEAD *tst, const char *key, int pos, int len);

/* Build a tre from a table. noffset is the offset to the key. */
void xp_tst_build(XP_TST_HEAD *tst, void *data, int elen, int noffset, int nelt);

/* Flush all of the nodes out of a tree. returns number of nodes removed. */
int xp_tst_flush(XP_TST_HEAD *tst, void (*freefunc)(void *data));

#endif /* xp_tst_h__ */

