#ident "%W%"
/*
 * xp_ifinfo.c - figure out interface information
 * Copyright (C) 2000 Rex Feany <laeos@laeos.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>
#include <netdb.h>
#include <sys/ioctl.h>
#include <errno.h>

#include "xp_malloc.h"
#include "xp_ifinfo.h"
#include "xpc.h"


/* These were written/stolen from W Richard Steven's "Unix Network Programming" */

struct xp_iflist *
xp_get_iflist(void)
{
	struct xp_iflist *ifi, *ifihead, **ifipnext;
	int sockfd, len, lastlen;
	char *ptr, *buf;
	struct ifconf ifc;
	struct ifreq *ifr, ifrcopy;
	struct sockaddr_in *sinptr;

	if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		return NULL;
	}

	lastlen = 0;
	len = 100 * sizeof(struct ifreq);	/* initial buffer size guess */
	for (;;) {
		buf = xp_malloc(len);
		ifc.ifc_len = len;
		ifc.ifc_buf = buf;
		if (ioctl(sockfd, SIOCGIFCONF, &ifc) < 0) {
			if (errno != EINVAL || lastlen != 0)
				xp_fatal_sys("ioctl(SIOCGIFCONF)");
		} else {
			if (ifc.ifc_len == lastlen)
				break;	/* success, len has not changed */
			lastlen = ifc.ifc_len;
		}
		len += 10 * sizeof(struct ifreq);	/* increment */
		xp_free(&buf);
	}
	ifihead = NULL;
	ifipnext = &ifihead;


	for (ptr = buf; ptr < buf + ifc.ifc_len;) {
		ifr = (struct ifreq *) ptr;

#ifdef	HAVE_SOCKADDR_SA_LEN
		len = MAX(sizeof(struct sockaddr), ifr->ifr_addr.sa_len);
#else
		switch (ifr->ifr_addr.sa_family) {
#ifdef	IPV6
		case AF_INET6:
			len = sizeof(struct sockaddr_in6);
			break;
#endif
		case AF_INET:
		default:
			len = sizeof(struct sockaddr);
			break;
		}
#endif				/* HAVE_SOCKADDR_SA_LEN */
		ptr += sizeof(ifr->ifr_name) + len;	/* for next one in buffer */

#ifdef	AF_LINK
		/* we don't care about link level things. just ip */
		if ( AF_LINK == ifr->ifr_addr.sa_family )
			continue;
#endif
		ifrcopy = *ifr;

		if (ioctl(sockfd, SIOCGIFFLAGS, &ifrcopy) < 0) {
			if (errno == ENXIO) /* device not configured */
				continue;
			xp_fatal_sys("ioctl(SIOCGIFFLAGS)");
		}

		if ((ifrcopy.ifr_flags & IFF_UP) == 0)
			continue;	/* ignore if interface not up */
		if (ifrcopy.ifr_flags & IFF_LOOPBACK)
			continue; 	/* ignore if ... */

		ifi = xp_malloc(sizeof(struct xp_iflist));
		*ifipnext = ifi;		/* prev points to this new one */
		ifipnext = &ifi->ifi_next;	/* pointer to next one goes here */

		memcpy(ifi->ifi_name, ifr->ifr_name, IFI_NAME);;
		ifi->ifi_name[IFI_NAME - 1] = '\0';
		getnameinfo(&ifr->ifr_addr, len, ifi->ifi_host, NI_MAXHOST, NULL, 0, NI_NUMERICSERV);

		switch (ifr->ifr_addr.sa_family) {
		case AF_INET:
			sinptr = (struct sockaddr_in *) &ifr->ifr_addr;
			if (ifi->ifi_addr == NULL) {
				ifi->ifi_addr = xp_malloc(sizeof(struct sockaddr_in));
				memcpy(ifi->ifi_addr, sinptr,
				       sizeof(struct sockaddr_in));
			}
			break;

		default:
			break;
		}
	}
	xp_free(&buf);
	return (ifihead);	/* pointer to first structure in linked list */
}


void 
xp_free_iflist (struct xp_iflist  *ifihead)
{
	struct xp_iflist *ifi, *ifinext;

	for (ifi = ifihead; ifi != NULL; ifi = ifinext) {
		if (ifi->ifi_addr != NULL)
			xp_free(&ifi->ifi_addr);
		ifinext = ifi->ifi_next;	/* can't fetch ifi_next after free() */
		xp_free(&ifi);			/* the ifi_info{} itself */
	}
}

