#ident "%W%"
/*
 * tst_search.c - search routines for tst
 * Copyright (C) 2000 Rex Feany <laeos@laeos.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_STDIO_H
# include <stdio.h>
#endif
#ifdef HAVE_CTYPE_H
# include <ctype.h>
#endif
#ifdef HAVE_ASSERT_H
# include <assert.h>
#endif

#include "xp_tst.h"
#include "xp_malloc.h"


/* get to a perticular node */
static struct xp_tst_node *
tst_find_node (XP_TST_HEAD *th, const char *str)
{
	unsigned char c ;
	struct xp_tst_node *p;

	assert(th);
	assert(str);
	assert(*str);

	p = XP_TST_BUCKET(th, str);

	while (p) {
		c = tolower(*str);

		if ( c < p->tn_split )
			p = p->tn_low;
		else if ( c > p->tn_split )
			p = p->tn_hi;
		else if (*++str == 0) 
			return p;
		else
			p = p->tn_eq;
	}
	return NULL;
}



/* Count the number of nodes below this one */
static int
tst_count (struct xp_tst_node *p)
{
	int count = 0;

	if ( p->tn_hi )
		count += tst_count(p->tn_hi);
	if ( p->tn_eq )
		count += tst_count(p->tn_eq);
	if ( p->tn_low )
		count += tst_count(p->tn_low);
	
	count += (p->tn_data != NULL);

	return count;
}


static void
do_partial (struct xp_tst_node *p, int *pos, int *len, struct xp_tst_array **data)
{
	/* short circut */
	if ( *len <= 0 )
		return;
	
	if ( p->tn_eq )
		do_partial(p->tn_eq, pos, len, data);

	if ( p->tn_data ) {

		if ( *pos ) {
			(*pos)--;
			return;
		}


		(*data)->data = p->tn_data;
		(*data)++;

		(*len)--;
	}

}

/* Finish a partial search */
static struct xp_tst_array *
finish_partial (struct xp_tst_node *p, int pos, int len)
{
	size_t l = len ? len : tst_count(p);
	struct xp_tst_array *data = xp_malloc( (l+1) * sizeof(struct xp_tst_array)  );
	struct xp_tst_array *retval = data;

	len = l;
	do_partial(p, &pos, &len, &data);

	retval[l].data = NULL;
	return retval;
}

/* Return a list of partial matches */
XP_TST_ARRAY *
xp_tst_partial(XP_TST_HEAD *th, const char *str, int pos, int len)
{
	struct xp_tst_node *p;

	if ( (p = tst_find_node(th, str)) ) {
		return finish_partial(p, pos, len);
	}
	return NULL;
}

/* Search for an entry in the tree */
void *
xp_tst_find (XP_TST_HEAD *th, const char *str)
{
	struct xp_tst_node *p;

	if ( (p = tst_find_node(th, str)) ) {
		return p->tn_data;
	}
	return NULL;
}
