#ident "%W%"
/*
 * xp_fatal.c -- fatal.
 * Copyright (C) 2000 Rex Feany <laeos@laeos.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_STDIO_H
# include <stdio.h>
#endif
#ifdef HAVE_STDARG_H
# include <stdarg.h>
#endif
#ifdef HAVE_ERRNO_H
# include <errno.h>
#endif
#ifdef HAVE_STRING_H
# include <string.h>
#endif

#include "xp_version.h"
#include "xpc.h"

static void fatal_msg(const char *m1, const char *m2) __attribute__((noreturn));

static void
fatal_msg(const char *m1, const char *m2)
{
	static int recursion = 0;

	if (recursion++)
		abort ();

	fputs ("\n\n\n\n", stderr);
	fputs (xp_version.v_web, stderr);
	fputs (": something went wrong", stderr);

	if (m1)
		fprintf(stderr, ": %s", m1);
	if (m2)
		fprintf(stderr, ": %s", m2);

	fputs ("\n\n", stderr);

	abort();
}


/* something bad happened. abort. */
void 
xp_fatal (const char *msg)
{
	fatal_msg(msg, NULL);
}

void 
xp_fatal_sys (const char *msg)
{
	fatal_msg(msg, strerror(errno));
}

